/**
 * JASMINe
 * Copyright (C) 2011 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CorrelateCollector.java 9079 2011-10-05 08:42:23Z durieuxp $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.probe.collectors.correlate.internal;

import org.ow2.jasmine.probe.JasmineIndicatorValue;
import org.ow2.jasmine.probe.JasmineIndicator;
import org.ow2.jasmine.probe.JasmineSingleNumberResult;
import org.ow2.jasmine.probe.JasmineSingleResult;
import org.ow2.jasmine.probe.collector.JasmineCollector;
import org.ow2.jasmine.probe.collector.JasmineCollectorException;
import org.ow2.jasmine.probe.collectors.JCollector;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

/**
 * Correlate Collector Implementation
 * @author durieuxp
 */
public class CorrelateCollector extends JCollector {

    private int ope;

    private Collection<JasmineCollector> sources;

    /**
     * Constructor
     * @param key
     * @param indic
     * @param period
     * @param ope
     * @param sources
     */
    public CorrelateCollector(String key, JasmineIndicator indic, int period, int ope, Collection<JasmineCollector> sources) {
        super(key, indic, period);
        this.ope = ope;
        this.sources = sources;
    }

    /**
     * return the last result for this indicator
     * @return JasmineIndicatorValue
     * @throws JasmineCollectorException
     */
    @Override
    public JasmineIndicatorValue getLastResult() throws JasmineCollectorException {
        logger.debug("");
        JasmineIndicatorValue jiv = new JasmineIndicatorValue();
        JasmineSingleResult jsr = new JasmineSingleNumberResult();
        HashMap<String, String> jsrProps = new HashMap<String, String>();
        long timestamp = System.currentTimeMillis();
        Number result = 0;
        Collection<JasmineSingleResult> jsrList = new ArrayList<JasmineSingleResult>();
        String domain = null;
        String server = null;
        String target = null;
        for (JasmineCollector elem : sources) {
            JasmineIndicatorValue jivs = elem.getLastResult();
            if (jivs == null) {
                logger.warn("No result available on source indicator {0}", elem.getName());
                return null;
            }
            if (jivs.getValues().isEmpty()) {
                logger.warn("No result available on source indicator {0}", elem.getName());
                return null;
            }
            if (jivs.getValues().size() > 1) {
                logger.warn("Correlation on multivalued indicator not supported");
                throw new JasmineCollectorException("Correlation on multivalued indicator not supported");
            }
            // Get the JSR (the only one)
            for (JasmineSingleResult jsr1 : jivs.getValues()) {
                HashMap<String, String> jsr1Props = jsr1.getProperties();
                jsrProps.putAll(jsr1Props);
                jsrList.add(jsr1);
                break;
            }
            // Analyse the JIV's target and meatadata domain and server
            String d = jivs.getMetadata().get("domain");
            if (d != null) {
                if (domain == null) {
                    domain = d;
                } else if (! d.equals(domain)) {
                    domain = "misc";
                }
            }
            String s = jivs.getMetadata().get("server");
            if (s != null) {
                if (server == null) {
                    server = s;
                } else if (! s.equals(server)) {
                    server = "misc";
                }
            }
            String t = jivs.getTarget();
            if (t != null) {
                if (target == null) {
                    target = t;
                } else if (! t.equals(target)) {
                    target = "misc";
                }
            }
        }
        if (jsrList.isEmpty()) {
            logger.warn("No result available for correlate indicator {0}", indicator.getName());
            return null;
        }
        switch (ope) {
            case CorrelateCollectorService.OP_ADD:
                result = addJsr(jsrList);
                break;
            case CorrelateCollectorService.OP_SUB:
                result = diffJsr(jsrList);
                break;
            case CorrelateCollectorService.OP_MUL:
                result = multiplyJsr(jsrList);
                break;
            case CorrelateCollectorService.OP_DIV:
                result = divideJsr(jsrList);
                break;
            case CorrelateCollectorService.OP_PERCENT:
                result = 100 * divideJsr(jsrList);
                break;
            default:
                throw new JasmineCollectorException("Bad correlate operation");
        }
        if (indicator.getScale() != 1) {
            result = divideValues(result, indicator.getScale());
        }
        jsr.setTimestamp(timestamp);
        jsr.setValue(result);
        jsr.setName(indicator.getName());
        jsr.setProperties(jsrProps);
        jiv.setName(indicator.getName());
        jiv.addValue(jsr);
        jiv.addMetadata("domain", domain);
        jiv.addMetadata("server", server);
        jiv.setTarget(target);
        return jiv;
    }

    /**
     * Stop polling
     */
    public void stopPolling() {
        logger.debug("");
        // Nothing to do
    }

    /**
     * Start polling
     */
    @Override
    public void startPolling() {
        logger.debug("");
        // nothing to do
    }

    // ------------------------------------------------------------------------------------
    // private methods
    // ------------------------------------------------------------------------------------

}
